@@ -36,7 +36,7 @@
 
 #define GROUP_SWITCHERS_GROUP "grp"
 #define DEFAULT_GROUP_SWITCH "grp:shift_caps_toggle"
-#define XMODMAP_KNOWN_FILE ".xmodmap"
+
 #define XMODMAP_CMD "xmodmap"
 
 const gchar GKBD_KEYBOARD_CONFIG_KEY_MODEL[] = "model";
@@ -49,6 +49,8 @@ const gchar *GKBD_KEYBOARD_CONFIG_ACTIVE[] = {
 	GKBD_KEYBOARD_CONFIG_KEY_OPTIONS
 };
 
+const gchar *XMODMAP_KNOWN_FILES[] = { ".xmodmap", ".Xmodmap" };
+
 /**
  * static common functions
  */
@@ -567,32 +569,48 @@ gkbd_keyboard_config_activate (GkbdKeyboardConfig * kbd_config)
 
 	/* Small bit of extensibility by using xmodmap */
 	if (rv) {
-		gchar *xmodmap_file = g_build_filename (g_get_home_dir (),
-							XMODMAP_KNOWN_FILE,
-							NULL);
-		if (g_file_test (xmodmap_file, G_FILE_TEST_EXISTS)) {
-			GError *error = NULL;
-			xkl_debug (150, "Loading custom xmodmap file %s\n",
-				   xmodmap_file);
-			gchar *command =
-			    g_strconcat (XMODMAP_CMD, " ", xmodmap_file,
-					 NULL);
-			/* Fire and forget - do not care about errors */
-			if (!g_spawn_command_line_async (command, &error)) {
-				xkl_debug (0,
-					   "Error loading custom xmodmap file: [%s]\n",
-					   error->message);
-				g_error_free (error);
-			}
-			g_free (command);
-		}
+		int i =
+		    sizeof (XMODMAP_KNOWN_FILES) /
+		    sizeof (XMODMAP_KNOWN_FILES[0]);
+		while (--i >= 0) {
+			gchar *xmodmap_file =
+			    g_build_filename (g_get_home_dir (),
+					      XMODMAP_KNOWN_FILES[i],
+					      NULL);
+			if (g_file_test (xmodmap_file, G_FILE_TEST_EXISTS)) {
+				GError *error = NULL;
+				xkl_debug (150,
+					   "Loading custom xmodmap file %s\n",
+					   xmodmap_file);
+				gchar *command =
+				    g_strconcat (XMODMAP_CMD, " ",
+						 xmodmap_file,
+						 NULL);
+				/* Fire and forget - do not care about errors */
+				if (!g_spawn_command_line_async
+				    (command, &error)) {
+					xkl_debug (0,
+						   "Error loading custom xmodmap file: [%s]\n",
+						   error->message);
+					g_error_free (error);
+				}
+				g_free (command);
 
-		g_free (xmodmap_file);
+				/* One file is enough */
+				i = 0;
+			}
+
+			g_free (xmodmap_file);
+		}
 	}
 
 	return rv;
 }
 
+/**
+ * gkbd_keyboard_config_start_listen:
+ * @func: (scope notified): a function to call when settings are changed
+ */
 void
 gkbd_keyboard_config_start_listen (GkbdKeyboardConfig * kbd_config,
 				   GCallback func, gpointer user_data)
@@ -705,6 +723,10 @@ gkbd_keyboard_config_to_string (const GkbdKeyboardConfig * config)
 	return result;
 }
 
+/**
+ * gkbd_keyboard_config_add_default_switch_option_if_necessary:
+ * Returns: (transfer full) (array zero-terminated=1): List of options
+ */
 gchar **
 gkbd_keyboard_config_add_default_switch_option_if_necessary (gchar **
 							     layouts_list,
